//
// File: test_trace_filters.cpp
// Created by: Olivier Langella
// Created on: 28/04/2019
//
/*******************************************************************************
 * Copyright (c) 2019 Olivier Langella <Olivier.Langella@u-psud.fr>.
 *
 * This file is part of the PAPPSOms++ library.
 *
 *     PAPPSOms++ is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     PAPPSOms++ is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with PAPPSOms++.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

// make test ARGS="-V -I 19,19"

#include <QDebug>
#include <QString>
#include <iostream>
#include <QFile>
#include <pappsomspp/precision.h>
#include <pappsomspp/msrun/output/mgfoutput.h>
#include <pappsomspp/processing/filters/filterresample.h>
#include <pappsomspp/processing/filters/filterpass.h>
#include <pappsomspp/processing/filters/filtermorpho.h>
#include <pappsomspp/processing/filters/filtertandemremovec13.h>
#include <pappsomspp/processing/filters/filterremovec13.h>
#include <pappsomspp/processing/filters/filtercomplementionenhancer.h>
#include <pappsomspp/psm/xtandem/xtandemhyperscore.h>
#include <pappsomspp/processing/filters/filtersuitestring.h>
#include <pappsomspp/processing/filters/filterchargedeconvolution.h>
#include "common.h"
#include "config.h"


using namespace pappso;
using namespace std;

int
main()
{
  qSetMessagePattern(QString("%{file}@%{line}, %{function}(): %{message}"));
  cout << endl << "..:: test trace filters ::.." << endl;
  //   OboPsiMod test;

  MassSpectrum spectrum_simple =
    readMgf(QString(CMAKE_SOURCE_DIR)
              .append("/test/data/peaklist_15046_simple_xt.mgf"));
  //.applyCutOff(150).takeNmostIntense(100).applyDynamicRange(100);
  cout << endl << "..:: FilterGreatestY ::.." << endl;

  qDebug();
  MassSpectrum most_intense(spectrum_simple);
  most_intense.filter(FilterGreatestY(10));

  if(most_intense.size() != 10)
    {
      cerr << "most_intense.size() != 10 " << most_intense.size() << endl;
      return 1;
    }

  qDebug() << __FILE__ << "@" << __LINE__ << __FUNCTION__ << "()";
  most_intense = spectrum_simple;
  most_intense.massSpectrumFilter(
    pappso::MassSpectrumFilterGreatestItensities(10));

  if(most_intense.size() != 10)
    {
      cerr << "most_intense.size() != 10 " << most_intense.size() << endl;
      return 1;
    }
  qDebug();
  most_intense =
    MassSpectrum(spectrum_simple)
      .massSpectrumFilter(pappso::MassSpectrumFilterGreatestItensities(1000));

  if(most_intense.size() != 100)
    {
      cerr << "most_intense.size() != 100 " << most_intense.size() << endl;
      return 1;
    }
  qDebug();
  most_intense = MassSpectrum(spectrum_simple).filter(FilterGreatestY(100));

  if(most_intense.size() != 100)
    {
      cerr << "most_intense.size() != 100 " << most_intense.size() << endl;
      return 1;
    }
  qDebug();
  // 3.018830
  cout << endl << "..:: resample filters ::.." << endl;
  Trace deca_suite({DataPoint(0, 1),
                    DataPoint(1, 2),
                    DataPoint(2, 3),
                    DataPoint(3, 4),
                    DataPoint(4, 3),
                    DataPoint(5, 2),
                    DataPoint(6, 1),
                    DataPoint(7, 1),
                    DataPoint(8, 2),
                    DataPoint(9, 3),
                    DataPoint(10, 4),
                    DataPoint(11, 5),
                    DataPoint(12, 6)});
  Trace deca_keep({DataPoint(9, 3), DataPoint(10, 4), DataPoint(11, 5)});

  Trace deca_remove({DataPoint(0, 1),
                     DataPoint(1, 2),
                     DataPoint(2, 3),
                     DataPoint(3, 4),
                     DataPoint(4, 3),
                     DataPoint(5, 2),
                     DataPoint(6, 1),
                     DataPoint(7, 1),
                     DataPoint(8, 2),
                     DataPoint(12, 6)});
  Trace deca_remove2({DataPoint(0, 1),
                      DataPoint(1, 2),
                      DataPoint(2, 3),
                      DataPoint(3, 4),
                      DataPoint(4, 3),
                      DataPoint(5, 2),
                      DataPoint(6, 1),
                      DataPoint(7, 1),
                      DataPoint(8, 2)});
  Trace deca_smaller({DataPoint(0, 1), DataPoint(1, 2), DataPoint(2, 3)});

  Trace deca_greater({DataPoint(10, 4), DataPoint(11, 5), DataPoint(12, 6)});
  Trace deca(deca_suite);
  deca.filter(FilterResampleRemoveXRange(9, 11.5));
  if(deca != deca_remove)
    {
      cerr << "deca != deca_remove " << deca.size() << endl;
      cerr << deca.toString().toStdString().c_str() << endl;
      return 1;
    }
  deca = deca_suite;
  deca.filter(FilterResampleRemoveXRange(9, 12));
  if(deca != deca_remove2)
    {
      cerr << "deca != deca_remove2 " << deca.size() << endl;
      cerr << deca.toString().toStdString().c_str() << endl;
      return 1;
    }

  deca = deca_suite;
  deca.filter(FilterResampleKeepXRange(9, 11));
  if(deca != deca_keep)
    {
      cerr << "deca != deca_keep " << deca.size() << endl;
      cerr << deca.toString().toStdString().c_str() << endl;
      return 1;
    }


  deca = deca_suite;
  deca.filter(FilterResampleKeepSmaller(3));
  if(deca != deca_smaller)
    {
      cerr << "deca != deca_smaller " << deca.size() << endl;
      cerr << deca.toString().toStdString().c_str() << endl;
      return 1;
    }

  deca = deca_suite;
  deca.filter(FilterResampleKeepGreater(9));
  if(deca != deca_greater)
    {
      cerr << "deca != deca_greater " << deca.size() << endl;
      cerr << deca.toString().toStdString().c_str() << endl;
      return 1;
    }
  cout << endl << "..:: morpho filters ::.." << endl;

  deca = deca_suite;
  deca.filter(FilterMorphoMedian(1));
  Trace deca_median({DataPoint(0, 2),
                     DataPoint(1, 2),
                     DataPoint(2, 3),
                     DataPoint(3, 3),
                     DataPoint(4, 3),
                     DataPoint(5, 2),
                     DataPoint(6, 1),
                     DataPoint(7, 1),
                     DataPoint(8, 2),
                     DataPoint(9, 3),
                     DataPoint(10, 4),
                     DataPoint(11, 5),
                     DataPoint(12, 6)});
  if(deca.size() != deca_median.size())
    {
      cerr << "deca.size() != deca_median.size() " << deca.size() << endl;
      cerr << deca.toString().toStdString().c_str() << endl;
      cerr << "origin" << endl
           << deca_median.toString().toStdString().c_str() << endl;
      return 1;
    }
  cout << "deca.size() == deca_median.size() " << deca.size() << endl;
  if(deca != deca_median)
    {
      cerr << "deca != deca_median " << deca.size() << endl;
      cerr << deca.toString().toStdString().c_str() << endl;
      cerr << "origin" << endl
           << deca_median.toString().toStdString().c_str() << endl;
      return 1;
    }

  cout << endl << "..:: FilterTandemDeisotope ::.." << endl;
  qDebug();
  MassSpectrum remove_tandem_deisotope =
    MassSpectrum(spectrum_simple)
      .massSpectrumFilter(FilterTandemDeisotope(1.5, 200));

  if(remove_tandem_deisotope.size() != 97)
    {
      cerr << "remove_tandem_deisotope.size() != 97 "
           << remove_tandem_deisotope.size() << endl;
      return 1;
    }
  qDebug();


  QualifiedMassSpectrum spectrum_scan_15968 = readQualifiedMassSpectrumMgf(
    QString(CMAKE_SOURCE_DIR).append("/test/data/scan_15968.mgf"));
  qDebug();
  QualifiedMassSpectrum spectrum_removed_tandem_deisotope =
    QualifiedMassSpectrum(spectrum_scan_15968).cloneMassSpectrumSPtr();
  qDebug();
  spectrum_removed_tandem_deisotope.getMassSpectrumSPtr()
    .get()
    ->massSpectrumFilter(FilterTandemDeisotope());
  qDebug();
  QFile file("scan_15968_tandem_deisotope.mgf");
  if(file.open(QIODevice::WriteOnly | QIODevice::Text))
    {
      MgfOutput mgf_output(&file);
      mgf_output.write(spectrum_removed_tandem_deisotope);
      mgf_output.close();
      file.close();
    }
  qDebug();

  Peptide pep15968("EITLGFVDLLR");
  qDebug();
  std::list<PeptideIon> ion_list;
  ion_list.push_back(PeptideIon::y);
  ion_list.push_back(PeptideIon::b);
  qDebug();

  cout << endl << "..:: FilterRemoveC13 ::.." << endl;
  qDebug();
  MassSpectrum remove_c13 =
    MassSpectrum(spectrum_simple)
      .filter(FilterRemoveC13(PrecisionFactory::getDaltonInstance(0.02)));

  if(remove_c13.size() != 93)
    {
      cerr << "remove_c13.size() != 93 " << remove_c13.size() << endl;
      return 1;
    }
  qDebug();

  QualifiedMassSpectrum spectrum_removed_c13 =
    QualifiedMassSpectrum(spectrum_scan_15968).cloneMassSpectrumSPtr();

  spectrum_removed_c13.getMassSpectrumSPtr().get()->filter(
    FilterRemoveC13(PrecisionFactory::getDaltonInstance(0.02)));

  QFile file2("scan_15968_remove_c13.mgf");
  if(file2.open(QIODevice::WriteOnly | QIODevice::Text))
    {
      MgfOutput mgf_output(&file2);
      mgf_output.write(spectrum_removed_c13);
      mgf_output.close();
      file2.close();
    }


  XtandemHyperscore hyperscore15968(
    *spectrum_scan_15968.getMassSpectrumSPtr().get(),
    pep15968.makePeptideSp(),
    2,
    PrecisionFactory::getDaltonInstance(0.02),
    ion_list,
    true);
  cout << "peptide " << pep15968.getSequence().toStdString()
       << " hyperscore15968=" << hyperscore15968.getHyperscore() << endl;
  cout << "peptide " << pep15968.getSequence().toStdString()
       << " mz1=" << pep15968.getMz(1) << " mz2=" << pep15968.getMz(2) << endl;

  XtandemHyperscore hyperscore15968_remove_c13(
    *spectrum_removed_c13.getMassSpectrumSPtr().get(),
    pep15968.makePeptideSp(),
    2,
    PrecisionFactory::getDaltonInstance(0.02),
    ion_list,
    true);
  cout << "peptide " << pep15968.getSequence().toStdString()
       << " hyperscore15968_remove_c13="
       << hyperscore15968_remove_c13.getHyperscore() << endl;
  cout << "peptide " << pep15968.getSequence().toStdString()
       << " mz1=" << pep15968.getMz(1) << " mz2=" << pep15968.getMz(2) << endl;


  cout << endl << "..:: FilterMassSpectrumComplementIonEnhancer ::.." << endl;
  qDebug();

  QualifiedMassSpectrum spectrum_complement_ion =
    QualifiedMassSpectrum(spectrum_scan_15968).cloneMassSpectrumSPtr();

  spectrum_complement_ion.getMassSpectrumSPtr().get()->filter(
    FilterComplementIonEnhancer(spectrum_complement_ion,
                                PrecisionFactory::getDaltonInstance(0.02)));

  QFile file3("scan_15968_complement_ion_enhancer.mgf");
  if(file3.open(QIODevice::WriteOnly | QIODevice::Text))
    {
      MgfOutput mgf_output(&file3);
      mgf_output.write(spectrum_complement_ion);
      mgf_output.close();
      file3.close();
    }

  XtandemHyperscore hyperscore15968_complement_ion_enhancer(
    *spectrum_complement_ion.getMassSpectrumSPtr().get(),
    pep15968.makePeptideSp(),
    2,
    PrecisionFactory::getDaltonInstance(0.02),
    ion_list,
    true);
  cout << "peptide " << pep15968.getSequence().toStdString()
       << " hyperscore15968_complement_ion_enhancer="
       << hyperscore15968_complement_ion_enhancer.getHyperscore() << endl;
  cout << "peptide " << pep15968.getSequence().toStdString()
       << " mz1=" << pep15968.getMz(1) << " mz2=" << pep15968.getMz(2) << endl;


  cout << endl << "..:: FilterGreatestYperWindow ::.." << endl;
  FilterGreatestYperWindow filter_per_window(40, 2);
  qDebug();
  QualifiedMassSpectrum spectrum_greatest_y_per_window =
    QualifiedMassSpectrum(spectrum_scan_15968).cloneMassSpectrumSPtr();
  qDebug();
  filter_per_window.filter(
    *spectrum_greatest_y_per_window.getMassSpectrumSPtr().get());
  qDebug();
  QFile file4("scan_15968_greatest_y_per_window.mgf");
  if(file4.open(QIODevice::WriteOnly | QIODevice::Text))
    {
      MgfOutput mgf_output(&file4);
      mgf_output.write(spectrum_greatest_y_per_window);
      mgf_output.close();
      file4.close();
    }
  qDebug();


  cout << endl
       << "..:: FilterGreatestYperWindow  on scan_PXD001468.mgf ::.." << endl;
  QualifiedMassSpectrum spectrum_scan_PXD001468 = readQualifiedMassSpectrumMgf(
    QString(CMAKE_SOURCE_DIR).append("/test/data/scan_PXD001468.mgf"));
  qDebug();
  QualifiedMassSpectrum spectrum_greatest_y_per_window_PXD001468 =
    QualifiedMassSpectrum(spectrum_scan_PXD001468).cloneMassSpectrumSPtr();
  qDebug();
  spectrum_greatest_y_per_window_PXD001468.getMassSpectrumSPtr().get()->filter(
    FilterGreatestYperWindow(40, 2));
  qDebug();
  QFile file5("scan_PXD001468_greatest_y_per_window.mgf");
  if(file5.open(QIODevice::WriteOnly | QIODevice::Text))
    {
      MgfOutput mgf_output(&file5);
      mgf_output.write(spectrum_greatest_y_per_window_PXD001468);
      mgf_output.close();
      file5.close();
    }
  qDebug();


  std::shared_ptr<pappso::FilterSuite> filter_suite =
    std::make_shared<pappso::FilterSuite>();
  filter_suite->push_back(std::make_shared<FilterGreatestYperWindow>(40, 2));
  filter_suite->push_back(std::make_shared<FilterRemoveC13>(
    PrecisionFactory::getDaltonInstance(0.02)));

  /*
    XtandemHyperscore hyperscore15968_greatest_y_per_window(
      *spectrum_greatest_y_per_window.getMassSpectrumSPtr().get(),
      pep15968.makePeptideSp(),
      2,
      PrecisionFactory::getDaltonInstance(0.02),
      ion_list,
      true);
    cout << "peptide " << pep15968.getSequence().toStdString()
         << " hyperscore15968_greatest_y_per_window="
         << hyperscore15968_greatest_y_per_window.getHyperscore() << endl;
    cout << "peptide " << pep15968.getSequence().toStdString()
         << " mz1=" << pep15968.getMz(1) << " mz2=" << pep15968.getMz(2) <<
    endl;

  */

  cout << endl << "..:: FilterSuiteString ::.." << endl;

  std::shared_ptr<pappso::FilterSuiteString> filter_suite_str =
    std::make_shared<pappso::FilterSuiteString>(
      "chargeDeconvolution|0.02dalton");


  cout << endl
       << "filter_suite_str = "
       << filter_suite_str.get()->toString().toStdString() << endl;

  cout << endl
       << "..:: FilterChargeDeconvolution from FilterSuiteString ::.." << endl;

  std::shared_ptr<pappso::FilterChargeDeconvolution>
    filter_charge_deconvolution =
      std::make_shared<pappso::FilterChargeDeconvolution>(
        filter_suite_str.get()->toString());

  Trace loaded_trace({// First peak z = 2
                      DataPoint(10, 20),
                      DataPoint(10.5, 18),
                      DataPoint(11, 16),
                      // Not a real peak
                      DataPoint(15, 50),
                      DataPoint(16, 51),
                      // Second peak z = 1
                      DataPoint(24, 200),
                      DataPoint(25, 20),
                      // Third peak z = 2
                      DataPoint(31, 20),
                      DataPoint(31.5, 15)});
  Trace theorical_trace({// Not modified peaks
                         DataPoint(15, 50),
                         DataPoint(16, 51),
                         // First peak merged
                         DataPoint(20 - MHPLUS, 54),
                         // Second
                         DataPoint(24, 220),
                         // Third
                         DataPoint(62 - MHPLUS, 35)});
  cout << "Transform Trace data" << endl;
  filter_charge_deconvolution->filter(loaded_trace);


  if(loaded_trace != theorical_trace)
    {
      cerr << "transformed Trace are different from theorical trace :" << endl;

      if(loaded_trace.xValues().size() == theorical_trace.xValues().size())
        {
          cerr << "transformed trace: \t";
          for(std::size_t i = 0; i < loaded_trace.xValues().size(); i++)
            {
              cerr << "(" << loaded_trace.xValues()[i] << ", "
                   << loaded_trace.yValues()[i] << ") ";
            }
          cerr << endl;
          cerr << "theorical trace : \t";
          for(std::size_t i = 0; i < theorical_trace.xValues().size(); i++)
            {
              cerr << "(" << theorical_trace.xValues()[i] << ", "
                   << theorical_trace.yValues()[i] << ") ";
            }
          cerr << endl;
        }
      else
        {
          cerr << "Number of peaks is different : "
               << loaded_trace.xValues().size() << " vs "
               << theorical_trace.xValues().size();
        }
      return 1;
    }


  cout << endl << "..:: FilterMorphoAntiSpike filters ::.." << endl;
  Trace anti_spike_trace({DataPoint(0, 0),
                          DataPoint(1, 1),
                          DataPoint(2, 3),
                          DataPoint(3, 0),
                          DataPoint(4, 0),
                          DataPoint(5, 0),
                          DataPoint(6, 0),
                          DataPoint(7, 0),
                          DataPoint(8, 0),
                          DataPoint(9, 3),
                          DataPoint(10, 0),
                          DataPoint(11, 0),
                          DataPoint(12, 0)});
  FilterMorphoAntiSpike filter_anti_spike(2);
  filter_anti_spike.filter(anti_spike_trace);

  if(anti_spike_trace[9].y != 0)
    {
      cerr << "anti_spike_trace.at(9) != 0";
      return 1;
    }
  if(anti_spike_trace[2].y != 3)
    {
      cerr << "anti_spike_trace.at(9) != 0";
      return 1;
    }
  cout << endl << anti_spike_trace.toString().toStdString() << endl;
  return 0;
}
